@@ -1,6 +1,6 @@
 # effect generic and methods; allEffects
 # John Fox and Jangman Hong
-#  last modified 2012-04-05 by J. Fox
+#  last modified 2012-08-28 by J. Fox
 
 effect <- function(term, mod, ...){
 	UseMethod("effect", mod)
@@ -80,86 +80,111 @@ effect <- function(term, mod, ...){
 #}
 
 effect.lm <- function (term, mod, xlevels = list(), default.levels = 10, given.values, 
-		se = TRUE, confidence.level = 0.95, transformation = list(link = family(mod)$linkfun, 
-				inverse = family(mod)$linkinv), typical = mean, ...){
-	if (missing(given.values)) 
-		given.values <- NULL
-	else if (!all(which <- names(given.values) %in% names(coef(mod)))) 
-		stop("given.values (", names(given.values[!which]), ") not in the model")
-	model.components <- analyze.model(term, mod, xlevels, default.levels)
-	predict.data <- model.components$predict.data
-	factor.levels <- model.components$factor.levels
-	factor.cols <- model.components$factor.cols
-	mod.aug <- model.components$mod.aug
-	term <- model.components$term
-	n.basic <- model.components$n.basic
-	x <- model.components$x
-	X.mod <- model.components$X.mod
-	cnames <- model.components$cnames
-	X <- model.components$X
-	formula.rhs <- formula(mod)[c(1, 3)]
-#	nrow.X <- nrow(X)
-#     mf <- model.frame(formula.rhs, data = rbind(X[, names(predict.data), 
-#                                                   drop = FALSE], predict.data), xlev = factor.levels)
-	Terms <- delete.response(terms(mod))
-	mf <- model.frame(Terms, predict.data, xlev = factor.levels)
-	mod.matrix <- model.matrix(formula.rhs, data = mf, contrasts.arg = mod$contrasts)
-#     mod.matrix <- mod.matrix.all[-(1:nrow.X), ]
-#     fit.1 <- na.omit(predict(mod))
-	wts <- mod$weights
-	if (is.null(wts)) 
-		wts <- rep(1, length(residuals(mod)))
-#     mod.2 <- lm.wfit(mod.matrix.all[1:nrow.X, ], fit.1, wts)
-#     class(mod.2) <- "lm"
-#     y <- if (inherits(mod, "glm")) 
-#         mod$y
-#     else na.omit(model.response(model.frame(mod)))
-#     discrepancy <- 100 * mean(abs(fitted(mod.2) - fit.1)/(1e-10 + 
-#         mean(abs(fit.1))))
-#     if (discrepancy > 0.001) 
-#         warning(paste("There is a discrepancy of", round(discrepancy, 
-#                                                          3), "percent \n     in the 'safe' predictions used to generate effect", 
-#                       term))
-	mod.matrix <- fixup.model.matrix(mod, mod.matrix, model.matrix(mod), 
-			X.mod, mod.aug, factor.cols, cnames, term, typical, given.values)
-	effect <- mod.matrix %*% mod$coefficients
-	result <- list(term = term, formula = formula(mod), response = response.name(mod), 
-			variables = x, fit = effect, x = predict.data[, 1:n.basic, 
-					drop = FALSE], model.matrix = mod.matrix, data = X, 
-			discrepancy = 0)
-	if (se) {
-		if (any(family(mod)$family == c("binomial", "poisson"))) {
-			dispersion <- 1
-			z <- qnorm(1 - (1 - confidence.level)/2)
+                       se = TRUE, confidence.level = 0.95, 
+                       transformation = list(link = family(mod)$linkfun, inverse = family(mod)$linkinv), 
+                       typical = mean, offset = mean, ...){
+    if (missing(given.values)) 
+        given.values <- NULL
+    else if (!all(which <- names(given.values) %in% names(coef(mod)))) 
+        stop("given.values (", names(given.values[!which]), ") not in the model")
+    off <- if (is.numeric(offset) && length(offset) == 1) offset
+    else if (is.function(offset)) {
+        mod.off <- model.offset(model.frame(mod))
+        if (is.null(mod.off)) 0 else offset(mod.off)
+    }
+    else stop("offset must be a function or a number")
+    model.components <- analyze.model(term, mod, xlevels, default.levels)
+    predict.data <- model.components$predict.data
+    factor.levels <- model.components$factor.levels
+    factor.cols <- model.components$factor.cols
+    mod.aug <- model.components$mod.aug
+    term <- model.components$term
+    n.basic <- model.components$n.basic
+    x <- model.components$x
+    X.mod <- model.components$X.mod
+    cnames <- model.components$cnames
+    X <- model.components$X
+    formula.rhs <- formula(mod)[c(1, 3)]
+    Terms <- delete.response(terms(mod))
+    mf <- model.frame(Terms, predict.data, xlev = factor.levels)
+    mod.matrix <- model.matrix(formula.rhs, data = mf, contrasts.arg = mod$contrasts)
+    wts <- mod$weights
+    if (is.null(wts)) 
+        wts <- rep(1, length(mod$residuals))
+    mod.matrix <- fixup.model.matrix(mod, mod.matrix, model.matrix(mod), 
+                                     X.mod, mod.aug, factor.cols, cnames, term, typical, given.values)
+    effect <- off + mod.matrix %*% mod$coefficients
+    result <- list(term = term, formula = formula(mod), response = response.name(mod), 
+                   variables = x, fit = effect, x = predict.data[, 1:n.basic, 
+                                                                 drop = FALSE], model.matrix = mod.matrix, data = X, 
+                   discrepancy = 0, offset=off)
+    if (se) {
+        if (any(family(mod)$family == c("binomial", "poisson"))) {
+            dispersion <- 1
+            z <- qnorm(1 - (1 - confidence.level)/2)
+        }
+        else {
+            dispersion <- sum(wts * mod$residuals^2)/mod$df.residual
+            z <- qt(1 - (1 - confidence.level)/2, df = mod$df.residual)
+        }
+        V2 <- dispersion * summary.lm(mod)$cov
+        V1 <- vcov(mod)
+        V <- if (inherits(mod, "fakeglm")) 
+            V1
+        else V2
+        vcov <- mod.matrix %*% V %*% t(mod.matrix)
+        rownames(vcov) <- colnames(vcov) <- NULL
+        var <- diag(vcov)
+        result$vcov <- vcov
+        result$se <- sqrt(var)
+        result$lower <- effect - z * result$se
+        result$upper <- effect + z * result$se
+        result$confidence.level <- confidence.level
+    }
+    if (is.null(transformation$link) && is.null(transformation$inverse)) {
+        transformation$link <- I
+        transformation$inverse <- I
+    }
+    result$transformation <- transformation
+    class(result) <- "eff"
+    result
+}
+
+summary.eff <- function(object, type=c("response", "link"), ...){
+	result <- list()
+	result$header <- paste("\n", gsub(":", "*", object$term), 'effect\n')
+	result$offset <- object$offset
+	type <- match.arg(type)
+	if (type == "response") {
+		object$fit <- object$transformation$inverse(object$fit)
+		if (!is.null(object$confidence.level)){
+			object$lower <- object$transformation$inverse(object$lower)
+			object$upper <- object$transformation$inverse(object$upper)
 		}
-		else {
-			dispersion <- sum(wts * mod$residuals^2)/mod$df.residual
-			z <- qt(1 - (1 - confidence.level)/2, df = mod$df.residual)
-		}
-#        mod.2$terms <- mod$terms
-		V2 <- dispersion * summary.lm(mod)$cov
-		V1 <- vcov(mod)
-		V <- if (inherits(mod, "fakeglm")) 
-					V1
-				else V2
-		vcov <- mod.matrix %*% V %*% t(mod.matrix)
-		rownames(vcov) <- colnames(vcov) <- NULL
-		var <- diag(vcov)
-		result$vcov <- vcov
-		result$se <- sqrt(var)
-		result$lower <- effect - z * result$se
-		result$upper <- effect + z * result$se
-		result$confidence.level <- confidence.level
 	}
-	if (is.null(transformation$link) && is.null(transformation$inverse)) {
-		transformation$link <- I
-		transformation$inverse <- I
+	result$effect <- array(object$fit,     
+			dim=sapply(object$variables, function(x) length(x$levels)),
+			dimnames=lapply(object$variables, function(x) x$levels))
+	if (!is.null(object$se)){
+		result$lower.header <- paste('\n Lower', round(100*object$confidence.level, 2), 
+				'Percent Confidence Limits\n')
+		result$lower <- array(object$lower,   
+				dim=sapply(object$variables, function(x) length(x$levels)),
+				dimnames=lapply(object$variables, function(x) x$levels))
+		result$upper.header <- paste('\n Upper', round(100*object$confidence.level, 2),
+				'Percent Confidence Limits\n')
+		result$upper <- array(object$upper,   
+				dim=sapply(object$variables, function(x) length(x$levels)),
+				dimnames=lapply(object$variables, function(x) x$levels))
 	}
-	result$transformation <- transformation
-	class(result) <- "eff"
+	if (object$discrepancy > 1e-3) result$warning <- paste("\nWarning: There is an average discrepancy of", 
+				round(object$discrepancy, 3),
+				"percent \n     in the 'safe' predictions for effect", object$term, '\n')
+	class(result) <- "summary.eff"
 	result
 }
 
+
 effect.gls <- function (term, mod, xlevels=list(), default.levels=10, given.values,
 		se=TRUE, confidence.level=.95, 
 		transformation=NULL, 
