@@ -1,4 +1,5 @@
 # effect.mer and effect.lme built from effect.lm by S. Weisberg 29 June 2011
+# last modified 2012-03-08 to require() lme4 or nlme. J. Fox
 
 
 # the function lm.wfit fit gets the hessian wrong for mer's.  Get the variance
@@ -89,18 +90,24 @@ vcov.fakeglm <- function(object, ...) object$vcov
 
 #The next four functions should be exported
 effect.mer <- function(term, mod, ...) {
+	if (!require(lme4)) stop("the lme4 package is not installed")
     result <- effect(term, mer.to.glm(mod), ...)
     result$formula <- as.formula(formula(mod))
     result
     }
     
 allEffects.mer <- function(mod, ...){
-  allEffects(mer.to.glm(mod), ...)}
+  	if (!require(lme4)) stop("the lme4 package is not installed")
+  	allEffects(mer.to.glm(mod), ...)
+}
  
 allEffects.lme <- function(mod, ...){
-  allEffects(lme.to.glm(mod), ...)}
+	if (!require(nlme)) stop("the nlme package is not installed")
+  	allEffects(lme.to.glm(mod), ...)
+}
   
 effect.lme <- function(term, mod, ...) {
+	if (!require(nlme)) stop("the nlme package is not installed")
     result <- effect(term, lme.to.glm(mod), ...)
     result$formula <- as.formula(formula(mod))
     result
